/*
 * This file is responsible for performing the logic of replacing
 * all occurrences of each mapped word with its emoji counterpart.
 */

/*global sortedEmojiMap*/

// emojiMap.js defines the 'sortedEmojiMap' variable.
// Referenced here to reduce confusion.


/**
 * Substitutes emojis into text nodes.
 * If the node contains more than just text (ex: it has child nodes),
 * call replaceText() on each of its children.
 *
 * @param  {Node} node    - The target DOM Node.
 * @return {void}         - Note: the emoji substitution is done inline.
 */


function replaceText (node) {
  // Setting textContent on a node removes all of its children and replaces
  // them with a single text node. Since we don't want to alter the DOM aside
  // from substituting text, we only substitute on single text nodes.
  // @see https://developer.mozilla.org/en-US/docs/Web/API/Node/textContent
  if (node.nodeType === Node.TEXT_NODE) {
    // This node only contains text.
    // @see https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType.

    // Skip textarea nodes due to the potential for accidental submission
    // of substituted emoji where none was intended.
    if (node.parentNode &&
        node.parentNode.nodeName === 'TEXTAREA') {
      return;
    }

    // Because DOM manipulation is slow, we don't want to keep setting
    // textContent after every replacement. Instead, manipulate a copy of
    // this string outside of the DOM and then perform the manipulation
    // once, at the end.
    let content = node.textContent;

    content = content.replaceAll("дь", "ď");
    content = content.replaceAll("зь", "ź");
    content = content.replaceAll("ль", "ľ");
    content = content.replaceAll("нь", "ń");
    content = content.replaceAll("ть", "ť");
    content = content.replaceAll("ць", "ć");
    content = content.replaceAll("сь", "ś");
    content = content.replaceAll("а", "a");
    content = content.replaceAll("б", "b");
    content = content.replaceAll("в", "v");
    content = content.replaceAll("г", "g");
    content = content.replaceAll("ґ", "g̀");
    content = content.replaceAll("д", "d");
    content = content.replaceAll("е", "e");
    content = content.replaceAll("ё", "ë");
    content = content.replaceAll("є", "ě");
    content = content.replaceAll("з", "z");
    content = content.replaceAll("і", "i");
    content = content.replaceAll("и", "i");
    content = content.replaceAll("й", "j");
    content = content.replaceAll("к", "k");
    content = content.replaceAll("л", "l");
    content = content.replaceAll("љ", "ľ");
    content = content.replaceAll("м", "m");
    content = content.replaceAll("н", "n");
    content = content.replaceAll("њ", "ń");
    content = content.replaceAll("о", "o");
    content = content.replaceAll("п", "p");
    content = content.replaceAll("р", "r");
    content = content.replaceAll("с", "s");
    content = content.replaceAll("т", "t");
    content = content.replaceAll("у", "u");
    content = content.replaceAll("ў", "ŭ");
    content = content.replaceAll("ф", "f");
    content = content.replaceAll("х", "h");
    content = content.replaceAll("ц", "c");
    content = content.replaceAll("ч", "č");
    content = content.replaceAll("ш", "š");
    content = content.replaceAll("щ", "šč");
    content = content.replaceAll("ы", "y");
    content = content.replaceAll("э", "è");
    content = content.replaceAll("ю", "ju");
    content = content.replaceAll("я", "ja");
    content = content.replaceAll("ж", "ž");

    content = content.replaceAll("Дь", "Ď").replaceAll("ДЬ", "Ď");
    content = content.replaceAll("Зь", "Ź").replaceAll("ЗЬ", "Ź");
    content = content.replaceAll("Ль", "Ľ").replaceAll("ЛЬ", "Ľ");
    content = content.replaceAll("Нь", "Ń").replaceAll("НЬ", "Ń");
    content = content.replaceAll("Ть", "Ť").replaceAll("ТЬ", "Ť");
    content = content.replaceAll("Ць", "Ć").replaceAll("ЦЬ", "Ć");
    content = content.replaceAll("Сь", "Ś").replaceAll("СЬ", "Ś");
    content = content.replaceAll("А", "A");
    content = content.replaceAll("Б", "B");
    content = content.replaceAll("В", "V");
    content = content.replaceAll("Г", "G");
    content = content.replaceAll("Д", "D");
    content = content.replaceAll("Е", "E");
    content = content.replaceAll("Ё", "Ë");
    content = content.replaceAll("Є", "Ě");
    content = content.replaceAll("З", "Z");
    content = content.replaceAll("І", "I");
    content = content.replaceAll("И", "I");
    content = content.replaceAll("Й", "J");
    content = content.replaceAll("К", "K");
    content = content.replaceAll("Л", "L");
    content = content.replaceAll("Љ", "Ľ")
    content = content.replaceAll("М", "M");
    content = content.replaceAll("Н", "N");
    content = content.replaceAll("Њ", "Ń");
    content = content.replaceAll("О", "O");
    content = content.replaceAll("П", "P");
    content = content.replaceAll("Р", "R");
    content = content.replaceAll("С", "S");
    content = content.replaceAll("Т", "T");
    content = content.replaceAll("У", "U");
    content = content.replaceAll("Ў", "Ŭ");
    content = content.replaceAll("Ф", "F");
    content = content.replaceAll("Х", "H");
    content = content.replaceAll("Ц", "C");
    content = content.replaceAll("Ч", "Č");
    content = content.replaceAll("Ш", "Š");
    content = content.replaceAll("Щ", "ŠČ");
    content = content.replaceAll("Ы", "Y");
    content = content.replaceAll("Э", "È");
    content = content.replaceAll("Ю", "Ju");
    content = content.replaceAll("Я", "Ja");
    content = content.replaceAll("Ж", "Ž");

    // Now that all the replacements are done, perform the DOM manipulation.
    node.textContent = content;
  }
  else {
    // This node contains more than just text, call replaceText() on each
    // of its children.
    for (let i = 0; i < node.childNodes.length; i++) {
      replaceText(node.childNodes[i]);
    }    
  }
}

// Start the recursion from the body tag.
replaceText(document.body);

// Now monitor the DOM for additions and substitute emoji into new nodes.
// @see https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver.
const observer = new MutationObserver((mutations) => {
  mutations.forEach((mutation) => {
    if (mutation.addedNodes && mutation.addedNodes.length > 0) {
      // This DOM change was new nodes being added. Run our substitution
      // algorithm on each newly added node.
      for (let i = 0; i < mutation.addedNodes.length; i++) {
        const newNode = mutation.addedNodes[i];
        replaceText(newNode);
      }
    }
  });
});
observer.observe(document.body, {
  childList: true,
  subtree: true
});
